/*********************************************************************************\

*     File:                  C O L O R . C P P                                    *
*                                                                                 *
*     Standard   C++                                                              *
*                                                                                 *
\*********************************************************************************/
/*
 * color.c - color allocation/sorting/freeing code
 *
 *
 *  Contains:
 *     void   SortColormap()
 *     void   ColorCompress8(trans)
 *     void   AllocColors()
 *     void   FreeColors()
 *     Status xvAllocColor()
 *     void   xvFreeColors()
 *     void   ApplyEditColor();
 *     int    MakeStdCmaps();
 *     void   MakeBrowCmap();
 *     void   ChangeCmapMode();
 */

#include "IOfile.h"



/********************************/
//------------------------------------------------------------------------------
// Upgrades the color table
//------------------------------------------------------------------------------


/************************************************/
/* structure and routine used in SortColormap() */
/************************************************/

typedef struct thing {
  byte r,g,b, n;          /* actual value of color + alignment */
  int oldindex;           /* its index in the old colormap */
  int use;                /* # of pixels of this color */
  int mindist;            /* min distance to a selected color */
} CMAPENT;



/***********************************/
void IOfile::SortColormap(const byte *pic, int pwide, int phigh, u_int *pnumcols,
                             const byte *rmap, const byte *gmap,const byte *bmap, 
                             byte *order, byte *trans)
{
  /* operates on 8-bit images.  sorts the colormap into 'best' order
   * 'order' is the 'best' order to allocate the colors.  'trans' is a 
   * transformation to be done to pic (in PIC8 mode) to compress the colormap
   */

  byte *p;
  u_int   i, j;
  int mdist, entry, d, hist[256];
  static CMAPENT c[256], c1[256], *cp, *cj, *ck;

  /* trans mappes the actual colormap in one without any repeated or unuded color 
     the old entry is the index, and the return number is the new equivalent entry
     new_entry_to_use = trans[old_entry] */


  /* init some stuff */
  for (i=0; i<256; i++) order[i]=i;

  /* initialize histogram and compute it */
  for (i=0; i<256; i++) hist[i]=0;
  for (i=pwide*phigh, p=(byte*)pic; i; i--, p++) hist[*p]++;
  
  if (DEBUG>1) {
    debug_printf("%s: Desired colormap\n",cmd);
    for (i=0; i<256; i++) 
      if (hist[i]) debug_printf("(%3d  %02x,%02x,%02x %d)\n",
         i,rmap[i],gmap[i],bmap[i], hist[i]);
    debug_printf("\n\n");
  }
  
  
  /* put the actually-used colors into the 'c' array in the order they occur
     also, while we're at it, calculate ncols, and close up gaps in
     colortable */
  
  /* we may have duplicate colors in the colormap, and we'd prefer not to.
  * order[0..numcols-1] contains the indices corresponding to the minimum colormap
  * set( r/g/bmap[order[0]] color must be the first color allocated). 
  * Trans array maps 'pic8' values (0-256) into corresponding non-repeated
  * colormap. 
  * To reflect the new colormap, note that order[updated_pix] entry and  
  * trans[old_pix] (both entrys in the old colormap) represent the same RGB value.
  */
  
  for (*pnumcols=i=0; i<256; i++) {
    if (hist[i]) {
      /* see if color #i is a duplicate used color */
      for (j=0; j<i; j++) { /* if color #j not used trans[j]==0 */
        if ( trans[j] && rmap[i] == rmap[j] && gmap[i] == gmap[j] && 
          bmap[i] == bmap[j]) break;
      }
      trans[i]    = (byte) j;
      if (j==i) {  /* unique color
        rmap[*pnumcols] = rmap[i];
        gmap[*pnumcols] = gmap[i];
        bmap[*pnumcols] = bmap[i]; */
        cp = &c[*pnumcols];
        cp->r = rmap[i];    cp->g = gmap[i];    cp->b = bmap[i];
        cp->use = hist[i];  cp->oldindex = i;
        cp->mindist = 1000000; /* 255^2 * 3 = 195075 */
        (*pnumcols)++;
      }else{ /* color is repeated -> merge repeated entry in the first one*/
        /* i = repeated entry, j = entry first happened */
        hist[j] += hist[i];
        hist[i] = (byte) 0;
      }
    }
    else //(hist[i])
      trans[i]  =(byte) 0;
  }

  /* Update c with the histogram without repeated colors */
  for (i=0; i<*pnumcols; i++) {
      c[i].use = hist[c[i].oldindex];
  }

  /* find most-used color, put that in c1[0] */
  entry = -1;  mdist = -1;
  for (i=0; i<*pnumcols; i++) {
    if (c[i].use > mdist) { mdist = c[i].use;  entry=i; }
  }
  xvbcopy((char *) &c[entry], (char *) &c1[0], sizeof(CMAPENT));
  c[entry].use = 0;   /* dealt with */
  
  
  /* sort rest of colormap.  Half of the entries are allocated on the
     basis of distance from already allocated colors, and half on the
     basis of usage.  (NB: 'taxicab' distance is used throughout this file.)

     Mod:  pick first 10 colors based on maximum distance.  pick remaining
     colors half by distance and half by usage   -- JHB

     To obtain O(n^2) performance, we keep each unselected color
     (in c[], with use>0) marked with the minimum distance to any of
     the selected colors (in c1[]).  Each time we select a color, we
     can update the minimum distances in O(n) time. 

     mod by Tom Lane   Tom.Lane@g.gp.cs.cmu.edu */

  for (i=1; i<*pnumcols; i++) {
    int ckR, ckG, ckB;
    /* Get RGB of color last selected  and choose selection method */
    ck = &c1[i-1];            /* point to just-selected color */
    ckR = ck->r; ckG = ck->g; ckB = ck->b;

    /* Now find the i'th most different color */

    if (i&1 || i<10) {
      /* select the unused color that has the greatest mindist */
      entry = -1;  mdist = -1;
      for (j=0, cj=c; j<*pnumcols; j++,cj++) {
        if (cj->use) {      /* this color has not been marked already */
          /* update mindist */
                d = (cj->r - ckR)*(cj->r - ckR) + 
              (cj->g - ckG)*(cj->g - ckG) + 
              (cj->b - ckB)*(cj->b - ckB);
                if (cj->mindist > d) cj->mindist = d;
          if (cj->mindist > mdist) { mdist = cj->mindist;  entry = j; }
        }
      }
    }
    else {
      /* select the unused color that has the greatest usage */
      entry = -1;  mdist = -1;
      for (j=0, cj=c; j<*pnumcols; j++,cj++) {
        if (cj->use) {  /* this color has not been marked already */
          /* update mindist */
                d = (cj->r - ckR)*(cj->r - ckR) + 
              (cj->g - ckG)*(cj->g - ckG) + 
                  (cj->b - ckB)*(cj->b - ckB);
                if (cj->mindist > d) cj->mindist = d;
          if (cj->use > mdist) { mdist = cj->use;  entry = j; }
        }
      }
    }


    /* c[entry] is the next color to put in the map.  do so */
    xvbcopy((char *) &c[entry], (char *) &c1[i], sizeof(CMAPENT));
    c[entry].use = 0;
  }
  

  for (i=0; i<*pnumcols; i++) order[i] = (byte) c1[i].oldindex;

  if (DEBUG>1) {
    debug_printf("%s: result of sorting colormap\n",cmd);
    for (i=0; i<*pnumcols; i++) 
      debug_printf("(%3d  %02x,%02x,%02x)     ",i,rmap[i],gmap[i],bmap[i]);
    debug_printf("\n\n");
    
    debug_printf("%s: allocation order table\n",cmd);
    for (i=0; i<*pnumcols; i++) 
      debug_printf("order[%d] = -> %d\n", i, order[i]);
    debug_printf("\n");
  }
}


/***********************************/
void IOfile::ColorCompress8(byte *pic, int pwide, int phigh, u_int nc,
                               byte *rmap, byte *gmap,byte *bmap,
                               byte *order, byte *trans)
{
 /* modify pic to reflect new (compressed) colormap 
  * To reflect the new colormap, note that order[updated_pix] entry and  
  * trans[old_pix] (both entrys in the old colormap) represent the same RGB value.
  */

  
  byte table[256];
  u_int i,disordered;

  //for (i=0; i<256; i++) table[i]=0;
  
  /* Solve the equation the equation order[updated_pix]==trans[old_pix] 
   * and store the values in table.
   * Trans maps repeated entries in unique and valid ones.
   *          color_entry(i) -> color_entry(trans[i])
   * Table is the inverse transformation of order (i is a valid index):
   *          order[table[i]]==i
   * For not valid indexes, table[i]==0
   * With any value of 'i' is certain: 
   *          order[table[trans[i]]]==trans[i] 
   */
  if (nc>256) 
   Error ("ColorCompress8: La imagen con mapa de colores tiene ms de 256 colores");

  for (i=0; i<nc; i++)
         table[order[i]]=i;
  for (i=0; i<256;i++)
      if(trans[i]!=i) table[i]=trans[i]; 
  
  /* if colortable ordered, don't do nothing */
  for (i=nc; i && table[i]==i; i--);
  if ( !i && !table[0])
     /*color in correct allocation order */
     disordered=FALSE;
  else
     /*colormap and pic must reflec the new order */
     disordered=TRUE;
  
  //for (i=0; i<256; i++) disordered|=(i==table[i]=order[trans[i]]);
  
  if (disordered) {
    byte *p, aux[256];
    
    for (i=0; i<nc; i++) aux[i]=rmap[order[i]];
    xvbcopy(aux,rmap,nc);
    for (i=0; i<nc; i++) aux[i]=gmap[order[i]];
    xvbcopy(aux,gmap,nc);
    for (i=0; i<nc; i++) aux[i]=bmap[order[i]];
    xvbcopy(aux,bmap,nc);

    for (i=pWIDE*pHIGH, p=pic; i; i--, p++) { *p = table[*p];}
  }
}

