/*********************************************************************************\
*     File:                  S A V E P I C . C P P                                *
*                                                                                 *
*     Standard   C++                                                              *
*                                                                                 *
\*********************************************************************************/
/* 
 * xvdir.c - Directory changin', file i/o dialog box
 *
 * callable functions:
 *
 *   DoSave()               -  calls appropriate save routines
 *   GetFullName()          -  sets the 'load/save-as' filename and default
 *   SetFullName()          -  gets the 'load/save-as' filename (no path)
 *   SetDirSaveMode()       -  sets default format/color settings
 */

#include "IOfile.h"

#include <io.h>
#include <ctype.h>
#include <stddef.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <direct.h>
#include <time.h>


#define MAXFNLEN 127               /* max len of filename being entered */

static char *saveColors[] = { "Full Color", 
            "Greyscale",
            "B/W Dithered",
            "Reduced Color" };

static char *saveFormats[] = { "GIF",
#ifdef HAVE_JPEG
             "JPEG",
#endif
#ifdef HAVE_TIFF
             "TIFF",
#endif
             "PostScript",
             "PBM/PGM/PPM (raw)",
             "PBM/PGM/PPM (ascii)",
             "X11 Bitmap",
             "XPM",
             "BMP",
             "Sun Rasterfile",
             "IRIS RGB",
             "Targa (24-bit)",
             "FITS",
             "PM",
             "RAW"
             };


/***************************************************/


int IOfile::SavePic(int filenum)
{
  FILE *fp;
  byte *thepic, *rp, *gp, *bp;
  u_int nc;
  int   rv, ptype, pfree=0;
  int   fmt, colType; /* format and color type with which the image is being saved */

  /* Should the pic be saved with a new name, we call GetFileName */
  if (filenum == SAVEPIC)  strncpy(filename,GetFileName(FALSE),MAXPATHLEN);
  
  /* Get save properties from actual status */
  fmt = GetFormatFromSuffix();
  colType = GetColorType();
  ptype=picType;
  
  /* opens file, does appropriate color pre-processing, calls save routine
     based on chosen format.  Returns '0' if successful */

  /* handle formats that pop up 'how do you want to save this' boxes */

#ifdef GS_PATH
  /* PostScript files not yet supported */
  if (fmt == F_PS) {      // PostScript 
    option = PSDialog();  // open PSDialog box
  }
#endif 

#ifdef HAVE_JPEG
  if (fmt == F_JPEG) {   /* JPEG */

    JPEGDialog();    /* open JPEGDialog box */
    if (colType == F_REDUCED || colType == F_MAXCOLORS) colType = F_FULLCOLOR;
    if (colType == F_BWDITHER) colType = F_GREYSCALE;
    writeJPEG(colType);       /* stuff for IG JPEG routines */ 
    return 0; 
  }
#endif

#ifdef HAVE_TIFF
  else if (fmt == F_TIFF) {   /* TIFF */
    TIFFDialog();            /* open TIFF Dialog box */
    if (colType == F_REDUCED) colType = F_FULLCOLOR;
  }
#endif


  WaitCursor();

  thepic = GetImage(&ptype, colType, &pfree, &nc, &rp, &gp, &bp,
                      MAX_ALLOCABLE_COLS);

  fp = OpenOutFile(filename);
  if (!fp) {
    /* Don't free thepic if its a pointer to the image of this object */
    if (pfree && thepic != pic ) {
        free(thepic); 
        thepic = NULL;
    }
    ArrowCursor();
    return -1;
  }

  rv = 0;

  switch (fmt) {
  case F_GIF:
    rv = WriteGIF   (fp, thepic, ptype, pWIDE, pHIGH, rp,gp,bp, nc,colType,picComments);
    break;
  #ifdef HAVE_TIFF
  case F_TIFF:
    rv = WriteTIFF(fp, thepic, ptype, pWIDE, pHIGH, rp,gp,bp, nc, colType,
                 filename, picComments);
    break;
  #endif
  case F_PM:
    rv = WritePM    (fp, thepic, ptype, pWIDE, pHIGH, rp,gp,bp, nc,colType,picComments);
    break;

  case F_PBMRAW:
    rv = WritePBM   (fp, thepic, ptype, pWIDE, pHIGH, rp,gp,bp, nc,colType,1,picComments);
    break;

  case F_PBMASCII: 
    rv = WritePBM   (fp, thepic, ptype, pWIDE, pHIGH, rp,gp,bp, nc,colType,0,picComments);
    break;
  
  case F_XBM:
    rv = WriteXBM   (fp, thepic, pWIDE, pHIGH, rp, gp, bp, filename);          break;
  
  case F_SUNRAS:
    rv = WriteSunRas(fp, thepic, ptype, pWIDE, pHIGH, rp, gp, bp, nc, colType,0);  break;

  case F_BMP:
    rv = WriteBMP   (fp, thepic, ptype, pWIDE, pHIGH, rp, gp, bp, nc, colType);    break;

  case F_IRIS:
    rv = WriteIRIS  (fp, thepic, ptype, pWIDE, pHIGH, rp, gp, bp, nc, colType);    break;
    
  case F_TARGA:
    rv = WriteTarga (fp, thepic, ptype, pWIDE, pHIGH, rp, gp, bp, nc, colType);    break;
    
  case F_XPM:
    rv = WriteXPM   (fp, thepic, ptype, pWIDE, pHIGH, rp, gp, bp, nc, colType, 
         filename, picComments);  break;  

  case F_FITS:
    rv = WriteFITS  (fp, thepic, ptype, pWIDE, pHIGH, rp, gp, bp, nc, colType, 
         picComments);    
    break;

  case F_RAW:
    rv = WriteRAW   (fp, thepic, ptype, pWIDE, pHIGH, rp, gp, bp);
    break;
  }
  

  CloseOutFile(fp, filename, rv) == 0;
  
  /* Don't free thepic if its a pointer to the image of this object */
  /* Colormaps pointed by rp, gp and bp musn't be freed because they are static
     colormaps. */
  if (pfree && thepic != pic ) {
     free(thepic); 
     thepic = NULL;
  }
  
  ArrowCursor();
  
  return rv;
}

/***************************************************/

void IOfile::SetFullName(char *st)
{
  strncpy(filename, st, (size_t) MAXPATHLEN-1);
  filename[MAXPATHLEN-1] = '\0';  /* make sure it's finished */
}

/***************************************************/

char *IOfile::GetFullName()
{
  return (filename);
}

/***************************************/
void IOfile::changeSuffix(int fmt, int colType)
{
  /* see if there's a common suffix at the end of the file name.  
     if there is, remember what case it was (all caps or all lower), lop
     it off, and replace it with a new appropriate suffix, in the
     same case */

  int allcaps;
  char *suffix, *sp, lowsuf[128];

  /* find the last '.' in the file name */
  suffix = (char *) strrchr(filename, '.');
  if (!suffix) {  /* add a '.' at the end of the name */
    suffix= filename+strlen(filename);
    *suffix='.';
    *(++suffix)='\0';
  }
  else suffix++;  /* point to first letter of the suffix */

  /* check for all-caposity */
  for (sp = suffix, allcaps=1; *sp; sp++) 
    if (islower(*sp)) allcaps = 0;
   
  switch (fmt) {
  case F_GIF:      strcpy(lowsuf,"gif");  break;
  case F_PM:       strcpy(lowsuf,"pm");   break;
  case F_PBMRAW:
  case F_PBMASCII: if (colType == F_FULLCOLOR || colType == F_REDUCED) 
                                                strcpy(lowsuf,"ppm");
                   else if (colType == F_GREYSCALE) strcpy(lowsuf,"pgm");
                   else if (colType == F_BWDITHER)  strcpy(lowsuf,"pbm");
                   break;

  case F_XBM:      strcpy(lowsuf,"xbm");  break;
  case F_SUNRAS:   strcpy(lowsuf,"ras");  break;
  case F_BMP:      strcpy(lowsuf,"bmp");  break;
  case F_PS:       strcpy(lowsuf,"ps");   break;
  case F_IRIS:     strcpy(lowsuf,"rgb");  break;
  case F_TARGA:    strcpy(lowsuf,"tga");  break;
  case F_XPM:      strcpy(lowsuf,"xpm");  break;
  case F_FITS:     strcpy(lowsuf,"fts");  break;
  case F_RAW:      strcpy(lowsuf,"raw");  break;

#ifdef HAVE_JPEG
  case F_JPEG:     strcpy(lowsuf,"jpg");  break;
#endif

#ifdef HAVE_TIFF
  case F_TIFF:     strcpy(lowsuf,"tif");  break;
#endif

  }

  if (allcaps) {  /* upper-caseify lowsuf */
    for (sp=lowsuf; *sp; sp++) 
      *sp = (islower(*sp)) ? toupper(*sp) : *sp;
  }  
  /* one other case:  if the original suffix started with a single
     capital letter, make the new suffix start with a single cap */
  if (isupper(suffix[0])) lowsuf[0] = toupper(lowsuf[0]);

  strcpy(suffix, lowsuf);   /* tack onto file name */
}
  



/***************************************/
/* Auxiliar colormaps */
static byte red[256], green[256], blue[256]; /* RGB colormap */

byte *IOfile::GetImage(int *PicTypeP, int  ColorType, int  *freeP, u_int  *ncP /*=NULL*/,
                            byte **rpp/*=NULL*/, byte **gpp/*=NULL*/, byte **bpp/*=NULL*/,
                            u_int MaxAllocCols/*=256*/)
{
  /* handles the whole ugly mess of the various save options.
   * returns an image, of type 'ptypeP', if (*ptypeP == PIC8),
   * also returns numcols 'ncP', and the r,g,b map
   * to use rmapP, gmapP, bmapP.
   * if freeP is set, image can safely be freed after it is saved 
   * NEVER RETURNS NULL
   */

  
  int   i;
  byte *pSave=NULL;  /* return default image if no transformation or a fail happens */
  
  /* given 'color mode' (F_FULLCOLOR, etc.), we may have to dither
     and/or use different colormaps.  Returns 'nc', rpp, gpp, bpp (the
     colormap to use).  Also, if the function returns non-NULL, it generated
     a new (dithered) image to use. */

  
  /* # # # #  B I C O L O R     M O N O C R O M E     I M A G E   # # # # */
  if (ColorType == F_BWDITHER) {
  
    /*  generate a BLACK & WHITE dithered version from 24 bits-RGB */
    if (picType == PIC24)  {                             

    #ifdef BW_DITHER_ALG
      byte *p24, *thepic;
      thepic = pic;
      p24 = GammifyPic24(thepic, pWIDE, pHIGH );
      if (p24) thepic = p24;
      /* generate a FSDithered 1-byte per pixel image */
      pSave = FSDither(thepic,picType, pWIDE, pHIGH, NULL,NULL,NULL, 0, 1);
      if (p24) free(p24);  /* won't need it any more */
      if (!pSave) 
       Error("Ha fallado el alojamiento de la imagen en B/N (GetImage)");
    #else   // BW_DITHER_ALG
      /* create a black & white mapped image*/
      pSave = (byte *) malloc((size_t) pWIDE * pHIGH);
      if (!pSave)
        FatalError("Ha fallado el alojamiento de la imagen bicolor (GetImage)");
      else {
        int val;
        static char *labels[] = {"\nOk"};
        static char buf[64] = "127";
        byte *p,*ep;
        val=InputDialog("Nivel umbral para el color blanco (1..254)", 
               buf,64, "0123456789",1);
        if ( val>255 || val < 0) val=127;
        for (i=pWIDE * pHIGH,p=pSave,ep=pic; i; i--, p++, ep+=3)
          *p = (byte)( ( MONO(ep[0],ep[1],ep[2]) > val) ? 1 : 0  );
      }
    #endif  // BW_DITHER_ALG
    }
    
    /*  generate a BLACK & WHITE version from colormapped & greyscaled image  */
    else if ( numcols > 2 || !mono ) { /* ptype == PIC8 and not monocrome*/
      /* if we're saving as 'dithered' and we aren't viewing as dithered
       and we're saving 'reduced' then generate a dithered image */
    #ifdef BW_DITHER_ALG
      pSave = FSDither(pic, picType, pWIDE, pHIGH, rMap,gMap,bMap, 0, 1);
      if (!pSave)
        FatalError("Ha fallado el alojamiento de la imagen en B/N (GetImage)");
    #else   // BW_DITHER_ALG
      /* create a black & white mapped image*/
      pSave = (byte *) malloc((size_t) pWIDE * pHIGH);
      if (!pSave)
        FatalError("Ha fallado el alojamiento de la imagen bicolor (GetImage)");
      else {
        int val;
        static char *labels[]={"\nOk"};
        static char buf[64] = "127";
        byte *p,*ep;
        val=InputDialog("Nivel umbral para el color blanco (1..254)",
               buf, 64, "0123456789",1);
        if ( val>255 || val < 0) val=127;
        for (i=255; i; i--)
            red [i] = (byte)( ( MONO(rMap[i],gMap[i],bMap[i]) > val) ? 1 : 0  );
        red [0] = (byte)( ( MONO(rMap[0],gMap[0],bMap[0]) > val) ? 1 : 0  );
          /* the random colormap has to be converted into a linear, shorted grayscale */
        for (i=pWIDE*pHIGH, p=pSave, ep=pic; i; i--, p++, ep++) *p = red[*ep];          
      }
    #endif  // BW_DITHER_ALG
    } else  { // IOfile real picture is monocromed
        pSave = pic;
    }
    red[0]=green[0]=blue[0]=0;     // black
    red[1]=green[1]=blue[1]=255;   // white
    //for (i=2; i<256; i++) blue[i]=green[i]=red[i]=i;
    //ColorType = F_BWDITHER; 
    *PicTypeP = PIC8;
    if (ncP != NULL)
      *ncP = 2;
  }

  /* # # # #  G R E Y S C A L E     I M A G E   # # # # */
  
  /*********** generate a GREY version ************/
  else if (ColorType == F_GREYSCALE) {     
    byte *p,*ep;
    if (picType == PIC24) {
      /* create a Greyscale mapped image*/
      pSave = (byte *) malloc((size_t) pWIDE * pHIGH);
      if (!pSave)
        FatalError("Ha fallado el alojamiento de la imagen en escala de grises (GetImage)");
      else {
        for (i=pWIDE * pHIGH,p=pSave,ep=pic; i; i--, p++, ep+=3)
          *p = MONO(ep[0],ep[1],ep[2]);
      }
    } else {  //PIC8    
      //if (numcols==256 && mono)  /* check direct GRAYSCALE */
      for (i=255; i && rMap [i]==i && gMap [i]==i && bMap [i] == i; i--);
        /* if i!=0, or color[0] != black, the colormap is not lineal */
      if ( i || rMap[0] || gMap[0] || bMap[0]) {
        /*ptype == PIC8 and not direct GREYSCALE*/
        /* create a Greyscale direct mapped image*/
        pSave = (byte *) malloc((size_t) pWIDE * pHIGH);
        if (!pSave)
          Error("Ha fallado el alojamiento de la imagen en escala de grises (GetImage)");
        else { 
          for (i=255; i; i--)
            red [i] = MONO(rMap[i], gMap[i], bMap[i]);
          red [0] = MONO(rMap[0], gMap[0], bMap[0]);
          /* the random colormap has to be converted into a linear, shorted grayscale */
          for (i=pWIDE*pHIGH, p=pSave, ep=pic; i; i--, p++, ep++) *p = red[*ep];          
        }
      } else { // IOfile real picture is greyscaled
        pSave = pic;
        mono=TRUE;
        numcols=256;
      }  
    } // PIC8
    for (i=255; i; i--)
      red [i] = green [i] = blue [i] = i;
    red [0] = green [0] = blue [0]= 0;
    //ColorType = F_GREYSCALE; 
    *PicTypeP = PIC8;
    if (ncP != NULL)
      *ncP = 256;
  } // end F_GREYSCALE

  /* # # # #  C O L O R M A P P E D     I M A G E   # # # # */
  
  else if ( (ColorType == F_REDUCED || ColorType == F_MAXCOLORS ) )  {
  /*********** generate a PIC8 version from PIC24 ***********/
    if (ncP == NULL || rpp == NULL || gpp == NULL || bpp == NULL)
        Error("Parmetros incorrectos en la llamada a IOfile::GetImage "
              "para transformar la imagen a mapa de color");

    if ( picType == PIC24 ) { 
      /* create a color mapped image*/
      pSave=Conv24to8(pic, pWIDE, pHIGH, red, green, blue, MaxAllocCols);
      /* Note: if pic is a 24 bit image, the colormap has been reduced */
      if (!pSave)
        Error("Ha fallado el alojamiento de la imagen con mapa de colores (GetImage)");    
      if (ncP != NULL)
        *ncP = MaxAllocCols;
    }  
    /*********** short PIC8 colormap ***********/
    else {  // picType==PIC8      
      
      // copy original colormap ( to avoid changing IOfile's one )
        xvbcopy((char *)rMap, (char *)red, 256);
        xvbcopy((char *)gMap, (char *)green,256);      
        xvbcopy((char *)bMap, (char *)blue,256);
      
      /* Reduce the number of colors if pic is a colormapped image */
      /* we have still duplicate colors in the colormap */
      if ( numcols>MaxAllocCols ) {
        /* Make a copy of IOfile's picture to avoid changing it */
        pSave = (byte *) malloc((size_t) pWIDE * pHIGH);
        if (!pSave)
          Error("Ha fallado el alojamiento de la imagen en escala de grises (GetImage)");
        else {
          xvbcopy(pic,pSave,(size_t) pWIDE * pHIGH);
          /* some stuff that necessary whenever running an algorithm or 
          installing a new 'pic' (or switching 824 modes) */
          byte trans[256];
          byte colAllocOrder[256];   /* order to allocate cols */
          if (ncP != NULL)
            *ncP=MaxAllocCols;
          // copy original colormap ( to avoid changing IOfile's one )
          xvbcopy((char *)rMap, (char *)red, 256);
          xvbcopy((char *)gMap, (char *)green,256);      
          xvbcopy((char *)bMap, (char *)blue,256);
          
          // if ncP = NULL, SortColormap doesn't work!
          u_int aux=256;
          SortColormap(pSave,pWIDE,pHIGH,(ncP != NULL) ? ncP : &aux,
                       red,green,blue,colAllocOrder,trans);
          ColorCompress8(pSave,pWIDE,pHIGH,(ncP != NULL) ? *ncP : aux,
                         red,green,blue,colAllocOrder,trans);
          /* colors are uniques! */
        }
      }
      else { // IOfile real picture is colormapped
        pSave = pic;
        if (ncP != NULL)
          *ncP = numcols; //Default value
      }
    } // end of PIC8
    //ColorType = F_REDUCED; 
    *PicTypeP = PIC8;  
  } // end of colormapped format

  /* # # # #  R G B - 2 4 bit    I M A G E   # # # # */
  
  else if (ColorType == F_FULLCOLOR) {

    /*********** generate a PIC24 version from PIC8 ***********/
    if ( picType == PIC8) { 
      /* convert color mapped image to a full color image */
      if ( numcols==256 && mono) 
          pSave=ConvGreyto24(pic,pWIDE,pHIGH);
        else
          pSave=Conv8to24(pic, pWIDE, pHIGH, rMap, gMap, bMap);
      if (!pSave)
        Error("Ha fallado el alojamiento de la imagen RGB (GetImage)");
    }

    else { // IOfile real picture is 24 bits RGB
       pSave = pic;
    }
    //ColorType == F_FULLCOLOR; 
    *PicTypeP = PIC24;
    if (ncP != NULL)
      *ncP = 0;
  }

  /* do not free image if pSave is the actual pic or is empty*/
  *freeP = (pic != pSave && pSave != NULL);

  /* Return static colormap if pointer has given*/
  if (rpp != NULL)
    *rpp = red;
  if (gpp != NULL)
    *gpp = green;
  if (bpp != NULL)
    *bpp = blue;     

  /* Default ( Never return NULL ) */
  if (pSave==NULL) { /* transform has failed. Return real type */ 
    *PicTypeP = picType;
    if (ncP != NULL)
      *ncP = numcols;
    // copy original colormap ( to avoid changing IOfile's one )
      xvbcopy((char *)rMap, (char *)red, 256);
      xvbcopy((char *)gMap, (char *)green,256);      
      xvbcopy((char *)bMap, (char *)blue,256);
    /*
    if (mono && numcols!=2) *ColorTypeP = F_GREYSCALE;
    else if (mono) *ColorTypeP = F_BWDITHER;
    else if (picType == PIC8) *ColorTypeP = F_REDUCED;
    else  *ColorTypeP = F_FULLCOLOR;
    */
    return pic;
  } 
  else return pSave;
}

     
/***************************************/
/***************************************/
FILE *IOfile::OpenOutFile( char *filename)
{
  /* opens file for output.  does various error handling bits.  Returns
     an open file pointer if success, NULL if failure */

  FILE *fp;
  struct stat st;

  if (!filename || filename[0] == '\0') return NULL;
  
  /* see if file exists (ie, we're overwriting) */
  if (stat(filename, &st)==0) {   /* stat succeeded, file must exist */
    /*static char *foo[] = { "\nOk", "\033Cancel" };
    char str[512];

    sprintf(str,"Overwrite existing file '%s'?", filename);
    if (PopUp(str, foo, 2)) return NULL;*/
    printf("Existing file \"%s\" overwrited", filename);
  }
    

  /* Open file */
  fp = fopen(filename, "wb");
  if (!fp) {
    char  str[512];
    sprintf(str,"Can't write file '%s'\n\n  %s.",filename, ERRSTR(errno));
    Error(str);
    return NULL;
  }

  return fp;
}
  

/***************************************/
int IOfile::CloseOutFile( FILE *fp, char *filename, int failed)
{
  char buf[64];

  /* close output file, and if piping, deal... Returns '0' if everything OK */

  if (failed) {    /* failure during format-specific output routine */
    char  str[512];
    sprintf(str,"Couldn't write file '%s'.", filename);
    Error(str);
    unlink(filename);   /* couldn't properly write file:  delete it */
    return 1;
  }
    
  if (fclose(fp) == EOF) {
    char  str[512];
    sprintf(str,"Can't close file '%s'\n\n  %s.",filename, ERRSTR(errno));
    Error(str);
    return 1;
  }

  buf[0]= '\0';  /* empty buffer */
  { /* compute size of written file */
    FILE *fp;
    long  filesize;
    fp = fopen(filename,"rb");
    if (fp) {
      fseek(fp, 0L, 2);
      filesize = ftell(fp);
      fclose(fp);

      sprintf(buf,"  (%ld bytes)", filesize);
    }
  }

  SetISTR(ISTR_INFO,"Successfully wrote '%s'%s", filename, buf);

  return 0;
}



/***************************************/
int IOfile::GetFormatFromSuffix(void)
{
  /* see if there's a common suffix at the end of the file name.  
     if there is, remember what case it was (all caps or all lower), lop
     it off, and replace it with a new appropriate suffix, in the
     same case */

  int allcaps;
  char *suffix, *sp, *dp, lowsuf[512];

  /* find the last '.' in the file name */
  suffix = (char *) strrchr(filename, '.');
  if (suffix == NULL ) {  /* Not supported save suffix or not present -> ask for it */

    /* Supose the same color format as in memory */
    int colType;
    if (mono && numcols>2) colType = F_GREYSCALE;
    else if (mono) colType = F_BWDITHER;
    else if (picType == PIC24) colType = F_FULLCOLOR;
    else if (picType == PIC8) colType = F_REDUCED;
    else colType = F_FULLCOLOR; /* Default */

    /* Ask for format */
    int format = PopMenu("Formatos de ficheros soportados",saveFormats,
                      sizeof(saveFormats)/sizeof(saveFormats[0]));

    /* Add a extension */
    changeSuffix(format, colType);

    return format;
  }
  else{  /* Have extension */
      suffix++;  /* point to first letter of the suffix */

      /* check for all-caposity */
      for (sp = suffix, allcaps=1; *sp; sp++)
        if (islower(*sp)) {
          allcaps = 0;
          break;
        }

      /* copy the suffix into an all-lower-case buffer */
      for (sp=suffix, dp=lowsuf; *sp; sp++, dp++) {
        *dp = (isupper(*sp)) ? tolower(*sp) : *sp;
      }
      *dp = '\0';

      /* compare for common suffixes */
      if (strcmp(lowsuf,"gif" )==0) return F_GIF;
      if (strcmp(lowsuf,"pm"  )==0) return F_PM;
      if (strcmp(lowsuf,"pbm" )==0) return F_PBMRAW;
      if (strcmp(lowsuf,"pgm" )==0) return F_PBMRAW;
      if (strcmp(lowsuf,"ppm" )==0) return F_PBMRAW;
      if (strcmp(lowsuf,"pnm" )==0) return F_PBMRAW;
      if (strcmp(lowsuf,"bm"  )==0) return F_XBM;
      if (strcmp(lowsuf,"xbm" )==0) return F_XBM;
      if (strcmp(lowsuf,"ras" )==0) return F_SUNRAS;
      if (strcmp(lowsuf,"bmp" )==0) return F_BMP;
      if (strcmp(lowsuf,"ps"  )==0) return F_PS;
      if (strcmp(lowsuf,"eps" )==0) return F_PS;
      if (strcmp(lowsuf,"rgb" )==0) return F_IRIS;
      if (strcmp(lowsuf,"tga" )==0) return F_TARGA;
      if (strcmp(lowsuf,"xpm" )==0) return F_XPM;
      if (strcmp(lowsuf,"fits")==0) return F_FITS;
      if (strcmp(lowsuf,"fts" )==0) return F_FITS;
    #ifdef HAVE_JPEG
      if (strcmp(lowsuf,"jpg" )==0) return F_JPEG;
      if (strcmp(lowsuf,"jpeg")==0) return F_JPEG;
      if (strcmp(lowsuf,"jfif")==0) return F_JPEG;
    #endif
    #ifdef HAVE_TIFF
      if (strcmp(lowsuf,"tif" )==0) return F_TIFF;
      if (strcmp(lowsuf,"tiff")==0) return F_TIFF;
    #endif
      if (strcmp(lowsuf,"raw" )==0) return F_RAW;
  }
  return F_BMP;
}

int IOfile::GetColorType(void)
{
    int filefmt = GetFormatFromSuffix();
    if ( (mono && numcols>2) || filefmt == F_FITS || filefmt == F_RAW ) 
      return F_GREYSCALE;
    if (mono || filefmt == F_XBM) return F_BWDITHER;
    if (picType == PIC8 && filefmt == F_XBM) return F_FULLCOLOR;
    if (picType == PIC8) return F_REDUCED;
    return F_FULLCOLOR; /* Default */
}

static char* ColorType[] = {"RGB color (full color)","Escala de Grises",
                          "Blanco y negro","Mapa de colores"};

int  IOfile::AskColorType(void)
{                   
   switch (PopMenu("Elige un formato de color",ColorType,
                   sizeof(ColorType)/sizeof(ColorType[0])))
   {
      case 0: return(F_FULLCOLOR); break;
      case 1: return(F_GREYSCALE); break;
      case 2: return(F_BWDITHER); break;
      case 3: return(F_REDUCED); break;
      default: return(F_FULLCOLOR);
   };
};

/**************************************
byte *IOfile::handleNormSel(int *pptype, int *pwide, int *phigh, int *pfree)
{
  /* called to return a pointer to a 'pic', its type, its width & height,
   * and whether or not it should be freed when we're done with it.  The 'pic'
   * returned is the desired portion of 'cpic' or 'epic' if there is a
   * selection, and the saveselCB is enabled, or alternately, it's the
   * whole cpic or epic.  
   * 
   * if selection does not intersect cpic/epic, returns cpic/epic
   * NEVER RETURNS NULL
   * 
   * Nowadays, the selection/crop function isn't developped

  *pptype = picType;
  *pwide = pWIDE;  *phigh = pHIGH;
  *pfree=0;
  return pic;
}
*/

void IOfile::ChangeImageFormat(int colType) 
{
  byte *thepic, *rp, *gp, *bp;
  int   pfree, NewPicType;
  u_int NewNumCols;
  /* format and color type with which the image is being saved */
  
  thepic = GetImage(&NewPicType, colType, &pfree, &NewNumCols, &rp, &gp, &bp,
                      MAX_ALLOCABLE_COLS);

  /* Change pic if a new one has returned */
  if (pfree) {
    StoreNewPic(thepic,NewPicType,
                ( (colType == F_GREYSCALE) || (colType == F_BWDITHER) )? TRUE : FALSE,
                pWIDE, pHIGH, NewNumCols, rp, gp, bp);
    RedrawPic(); //Redraw Image.
  }
  else   if (thepic != NULL && thepic != pic && colType != F_FULLCOLOR 
              && rp != NULL && gp != NULL && bp != NULL && NewPicType==PIC8 
              && rp != rMap && gp != gMap && bp != bMap ) {

    // Copy colormap (pic is the same but colormap has changed).
    xvbcopy((char *)rp, (char *)rMap, 256);
    xvbcopy((char *)gp, (char *)gMap,256);      
    xvbcopy((char *)bp, (char *)bMap,256);

    picType = NewPicType;
    mono =    (colType == F_GREYSCALE) || (colType == F_BWDITHER) ;
    numcols = NewNumCols;
    RedrawPic(); //Redraw Image.
  }
};






