/*********************************************************************************\

*     File:                  T I F F W R . C P P                                  *
\*********************************************************************************/

/*
 * tiffwr.cpp - write routine for TIFF pictures
 *
 * WriteTIFF(fp,pic,w,h,r,g,b,numcols,style,raw,fname,comment)
 */

#define NEEDSARGS

#include "IOfile.h"

#ifdef HAVE_TIFF

#include "..\\tiff\\tiffio.h"    /* has to be after xv.h, as it needs varargs/stdarg */



static int TIFFcomp=COMPRESSION_NONE;

static void setupColormap   (TIFF *, byte *, byte *, byte *);

/*********************************************/
void IOfile::SetTIFFcompression(int value)
{
  TIFFcomp=value;
}
int  IOfile::GetTIFFcompression(void)
{
  return TIFFcomp;
}


/*********************************************/
static void setupColormap(TIFF *tif, byte *rmap, byte *gmap, byte *bmap)
{
  short red[256], green[256], blue[256];
  int i;
  
  /* convert 8-bit colormap to 16-bit */
  for (i=0; i<256; i++) {
#define  SCALE(x)  ((((int)x)*((1L<<16)-1))/255)
    red[i] = SCALE(rmap[i]);
    green[i] = SCALE(gmap[i]);
    blue[i] = SCALE(bmap[i]);
  }
  TIFFSetField(tif, TIFFTAG_COLORMAP, red, green, blue);
}



/*******************************************/
int IOfile::WriteTIFF(FILE *fp, byte *pic,int ptype,int w,int h, byte *rmap, 
                     byte *gmap, byte *bmap,int numcols,int colorstyle,
                     char *fname,char *comment)
{
  TIFF *tif;
  byte *pix;
  int   i,j;

  tif = TIFFOpen(fname, "wb");

  if (!tif) return 0;

  WaitCursor();

  TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, w);
  TIFFSetField(tif, TIFFTAG_IMAGELENGTH, h);
  TIFFSetField(tif, TIFFTAG_COMPRESSION, TIFFcomp);

  if (comment && strlen(comment) > (size_t) 0) {
    TIFFSetField(tif, TIFFTAG_IMAGEDESCRIPTION, comment);
  }

  if (TIFFcomp == COMPRESSION_CCITTFAX3)
      TIFFSetField(tif, TIFFTAG_GROUP3OPTIONS,
    GROUP3OPT_2DENCODING+GROUP3OPT_FILLBITS);

  TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
  TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 1);
  TIFFSetField(tif, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);
  TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, h);

  TIFFSetField(tif, TIFFTAG_RESOLUTIONUNIT, (int)2);
  TIFFSetField(tif, TIFFTAG_XRESOLUTION, (float)1200.0);
  TIFFSetField(tif, TIFFTAG_YRESOLUTION, (float)1200.0);


  /* write the image data */

  if (ptype == PIC24) {  /* only have to deal with FULLCOLOR or GREYSCALE */
    if (colorstyle == F_FULLCOLOR) {
      TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 3);
      TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE,   8);
      TIFFSetField(tif, TIFFTAG_PHOTOMETRIC,     PHOTOMETRIC_RGB);
      
      TIFFWriteEncodedStrip(tif, 0, pic, w*h*3);
    }

    else {  /* colorstyle == F_GREYSCALE */
      byte *tpic, *tp, *sp;

      TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 1);
      TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE,   8);
      TIFFSetField(tif, TIFFTAG_PHOTOMETRIC,    PHOTOMETRIC_MINISBLACK);

      tpic = (byte *) malloc((size_t) w*h);
      if (!tpic) FatalError("unable to malloc in WriteTIFF()");

      for (i=0, tp=tpic, sp=pic; i<w*h; i++, sp+=3) 
        *tp++ = MONO(sp[0],sp[1],sp[2]);
      
      TIFFWriteEncodedStrip(tif, 0, tpic, w*h);

      free(tpic);
    }
  }

  else {  /* PIC8 */
    if (colorstyle == F_FULLCOLOR) {                  /* 8bit Palette RGB */
      TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8);
      TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_PALETTE);
      setupColormap(tif, rmap, gmap, bmap);
      TIFFWriteEncodedStrip(tif, 0, pic, w*h);
    }

    else if (colorstyle == F_GREYSCALE) {             /* 8-bit greyscale */
      byte rgb[256];
      byte *tpic = (byte *) malloc((size_t) w*h);
      byte *tp = tpic;
      TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8);
      TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
      for (i=0; i<numcols; i++) rgb[i] = MONO(rmap[i],gmap[i],bmap[i]);
      for (i=0, pix=pic; i<w*h; i++,pix++) {
        if ((i&0x7fff)==0) WaitCursor();
        *tp++ = rgb[*pix];
      }
      TIFFWriteEncodedStrip(tif, 0, tpic, w*h);
      free(tpic);
    }

    else if (colorstyle == F_BWDITHER) {             /* 1-bit B/W stipple */
      int bit,k,flipbw;
      byte *tpic, *tp;

      flipbw = (MONO(rmap[0],gmap[0],bmap[0]) > MONO(rmap[1],gmap[1],bmap[1]));
      TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 1);
      TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
      tpic = (byte *) malloc((size_t) TIFFStripSize(tif));
      tp = tpic;
      for (i=0, pix=pic; i<h; i++) {
        if ((i&15)==0) WaitCursor();
        for (j=0, bit=0, k=0; j<w; j++, pix++) {
          k = (k << 1) | *pix;
          bit++;
          if (bit==8) {
            if (flipbw) k = ~k;
            *tp++ = (byte) (k&0xff);
            bit = k = 0;
    }
  } /* j */
  if (bit) {
    k = k << (8-bit);
    if (flipbw) k = ~k;
    *tp++ = (byte) (k & 0xff);
  }
      }
      TIFFWriteEncodedStrip(tif, 0, tpic, TIFFStripSize(tif));
      free(tpic);
    }
  }


  TIFFClose(tif);

  return 0;
}

/***************************************************/

#endif // HAVE_TIFF
